// name: CTab.h
// author: J. Michael Word
// date written: 7/15/97
// purpose: Interface for PDB atom connection table

// **************************************************************
// NOTICE: This is free software and the source code is freely
// available. You are free to redistribute or modify under the
// conditions that (1) this notice is not removed or modified
// in any way and (2) any modified versions of the program are
// also available for free.
//               ** Absolutely no Warranty **
// Copyright (C) 1999 J. Michael Word
// **************************************************************

#ifndef CTAB_H
#define CTAB_H 1

#ifdef OLD_STD_HDRS
#include <stdio.h>
#else
#include <cstdio>
#endif

#include "AtomConn.h"
#include "Seq.h"
#include "Stringclass.h"
#include "Dict.h"

// -----------------------------------------
//  atom connections for a given residue
// -----------------------------------------
class ResConn : public Dict<Stringclass, AtomConn> {
public:
         ResConn(int sz_est) : Dict<Stringclass, AtomConn>(sz_est) {}
virtual ~ResConn() {}

   bool put(AtomConn *value) {
      return Dict<Stringclass, AtomConn>::put(value->name(), value);
   }
   AtomConn* get(const Stringclass &atomname) const {
      return Dict<Stringclass, AtomConn>::get(atomname);
   }
   atomPlacementPlan* planHplacement(const Stringclass &atomname) const;
   Seq<atomPlacementPlan> genHplans() const;

private:
   ResConn(const ResConn&);            // can't copy
   ResConn& operator=(const ResConn&); // can't assign
};

// -----------------------------------------
//  a specific location in an open file
// -----------------------------------------
class FileLoc {
public:
   FileLoc(FILE *stream, int xtra) : _value(xtra) {
      _location = ::ftell(stream);
   }
   virtual ~FileLoc() {}

   bool relocate(FILE *stream) const{
      return ::fseek(stream, _location, SEEK_SET) == 0;
   }
   int value() { return _value; }
private:

   long _location; // position in file (bytes from the beginning)
   int  _value;    // xtra info. (used to store table size)
};

// -------------------------------------------------------------
//  find residue connection tables in a connection database file
// -------------------------------------------------------------
class CTab {
public:
         CTab(const Stringclass& dbfile, int szest);
virtual ~CTab() { if (_fp) ::fclose(_fp); }

   bool ok() const { return _fp != NULL; }
   const Stringclass& filename() const { return _filename; }

   ResConn* findTable(const Stringclass &resname); // note: updates a cache...

   int numConn(const Stringclass &resname, const Stringclass &atomname); // note: updates a cache...

private:
   CTab(const CTab&);            // can't copy
   CTab& operator=(const CTab&); // can't assign

   Stringclass                 _filename;
   FILE *                 _fp;
   Dict<Stringclass, FileLoc>  _filedict;
   Dict<Stringclass, ResConn>  _rescache;

   static const int DBbufsz;
};

#endif
