// name: CliqueList.h
// author: J. Michael Word
// date written: 2/7/98
// purpose: Sequences containing collections of motion pointers
//          which refer to interacting residues, along with singletons

// **************************************************************
// NOTICE: This is free software and the source code is freely
// available. You are free to redistribute or modify under the
// conditions that (1) this notice is not removed or modified
// in any way and (2) any modified versions of the program are
// also available for free.
//               ** Absolutely no Warranty **
// Copyright (C) 1999 J. Michael Word
// **************************************************************

#ifndef CLIQUELIST_H
#define CLIQUELIST_H 1

#ifdef OLD_STD_HDRS
#include <iostream.h>
#else
#include <iostream>
#endif

#include "Seq.h"
#include "List.h"
#include "Stringclass.h"

class Mover;
typedef Mover* MoverPtr;

class CliqueList {
public:
   CliqueList() {}
  ~CliqueList() {}

   CliqueList(const CliqueList& cl) : _cliques(cl._cliques),
                                      _singles(cl._singles) {}

   CliqueList& operator=(const CliqueList& cl) {
      if (this != &cl) {
	 _cliques = cl._cliques; _singles = cl._singles;
      }
      return *this;
   }

   void insertClique(const Seq<MoverPtr>& lst) { _cliques.insert(lst); }
   void insertSingleton(const MoverPtr    ptr) { _singles.insert(ptr); }

   void sortSingletonsByDescr();

   Seq< Seq<MoverPtr> > cliques() const { return _cliques; }
          Seq<MoverPtr> singles() const { return _singles; }

   int numCliques()    { return _cliques.len(); }
   int numSingletons() { return _singles.len(); }

   void describe(ostream& os) const;
   void formatSingles(List<Stringclass>& cliqueNotes) const;
   void formatClique(List<Stringclass>& cliqueNotes, int c) const;
private:
   Seq< Seq<MoverPtr> > _cliques;
   Seq<MoverPtr>        _singles;
};
#endif
