// name: ResBlk.h
// author: J. Michael Word
// date written: 7/15/97
// purpose: The records for a single residue and assoc. info

// **************************************************************
// NOTICE: This is free software and the source code is freely
// available. You are free to redistribute or modify under the
// conditions that (1) this notice is not removed or modified
// in any way and (2) any modified versions of the program are
// also available for free.
//               ** Absolutely no Warranty **
// Copyright (C) 1999 J. Michael Word
// **************************************************************

#ifndef RESBLK_H
#define RESBLK_H 1

#include "PDBrec.h"
#include "ListIter.h"
#include "MultiDict.h"
#include "Stringclass.h"

bool sameres(const PDBrec &r1, const PDBrec &rn);

// ----------------
//  A residue block
// ----------------
class ResBlk {
public:
   ResBlk(NonConstListIter<PDBrec>& lit);
  ~ResBlk() {}

   bool valid() const { return _insertPt.valid() && _resAtoms.size() > 0; }

   unsigned long size() const { return _resAtoms.size(); }

   const PDBrec& firstRec() const { return _firstRec; }

   void addPrevRec(const PDBrec& rec) {
      _resAtoms.put(Stringclass("-") + fixupAtomKey(rec), rec);
   }
   void addNextRec(const PDBrec& rec) {
      _resAtoms.put(Stringclass("+") + fixupAtomKey(rec), rec);
   }

   Seq<PDBrec> get(const Stringclass& atomname) const {
      return _resAtoms.get(atomname);
   }

   bool contains(const Stringclass& atomname) const {
      return _resAtoms.exists(atomname);
   }

   bool insertNewRec(const PDBrec& rec) { // add a new atom to the list
      bool successflag = _insertPt.insertAfter(rec);
      if (successflag) { _insertPt++; }
      return successflag;
   }

   MultiDictIterator<Stringclass, PDBrec> atomIt() const {
      return MultiDictIterator<Stringclass, PDBrec>(_resAtoms);
   }
private:
   ResBlk(const ResBlk&);            // can't copy
   ResBlk& operator=(const ResBlk&); // can't assign

   Stringclass fixupAtomKey(const PDBrec &r) const;

   PDBrec                    _firstRec; // the first atom or het record
   MultiDict<Stringclass, PDBrec> _resAtoms; // all of the atoms by atomname
   NonConstListIter<PDBrec>  _insertPt; // where we put new atom records
};

#endif
